# Detect the distribution in use
%global __despace head -n 1 | tr -d '[:space:]' | sed -e 's/[(].*[)]//g'
%global __lower4 cut -c 1-4 | tr '[:upper:]' '[:lower:]'
%global __distfile %([ -f /etc/SuSE-release ] && echo /etc/SuSE-release || echo /etc/redhat-release)
%global __distinit %(sed -e 's/ release .*//' -e 's/\\([A-Za-z]\\)[^ ]*/\\1/g' %{__distfile} | %{__despace} | %{__lower4})
%global __distvers %(sed -e 's/.* release \\([^. ]*\\).*/\\1/' %{__distfile} | %{__despace})
# Identify CentOS Linux and Scientific Linux as rhel
%if "%{__distinit}" == "c" || "%{__distinit}" == "cl" || "%{__distinit}" == "sl" || "%{__distinit}" == "sls"
%global __distinit rhel
%endif
# Dist tag for Fedora is still "fc"
%if "%{__distinit}" == "f"
%global __distinit fc
%endif

# Perl and RPM versioning don't work the same way
%global extraversion %{nil}

# Provides/Requires filtering is different from rpm 4.9 onwards
%global rpm49 %(rpm --version | perl -p -e 's/^.* (\\d+)\\.(\\d+).*/sprintf("%d.%03d",$1,$2) ge 4.009 ? 1 : 0/e')

Name:		perl-Coro
Version:	6.48
Release:	1.%{__distinit}%{__distvers}
Summary:	The only real threads in perl
License:	GPL+ or Artistic
Group:		Development/Libraries
URL:		http://search.cpan.org/dist/Coro/
Source0:	http://search.cpan.org/CPAN/authors/id/M/ML/MLEHMANN/Coro-%{version}%{?extraversion}.tar.gz
Patch0:		Coro-5.25-ucontext-default.patch
Patch1:		Coro-6.0-jitpath.patch
Patch2:		Coro-6.47-perl58.patch
BuildRoot:	%{_tmppath}/%{name}-%{version}-%{release}-root-%(id -nu)
# Module Build (perl-devel split from main perl package at F-7)
BuildRequires:	coreutils
BuildRequires:	findutils
BuildRequires:	gcc
BuildRequires:	make
BuildRequires:	perl
%if 0%{?fedora} > 6 || 0%{?rhel} > 5
BuildRequires:	perl-devel
%endif
BuildRequires:	perl(Canary::Stability)
BuildRequires:	perl(ExtUtils::MakeMaker)
BuildRequires:	/usr/include/valgrind/valgrind.h
# Module Runtime
BuildRequires:	perl(base)
BuildRequires:	perl(AnyEvent) >= 7
BuildRequires:	perl(AnyEvent::AIO) >= 1
BuildRequires:	perl(AnyEvent::BDB) >= 1
BuildRequires:	perl(AnyEvent::DNS)
BuildRequires:	perl(AnyEvent::Socket)
BuildRequires:	perl(AnyEvent::Util)
BuildRequires:	perl(BDB)
BuildRequires:	perl(Carp)
BuildRequires:	perl(common::sense)
BuildRequires:	perl(Config)
BuildRequires:	perl(Errno)
BuildRequires:	perl(EV) >= 4
BuildRequires:	perl(Event) >= 1.08
BuildRequires:	perl(Exporter)
BuildRequires:	perl(Guard) >= 0.5
BuildRequires:	perl(IO::AIO) >= 3.1
BuildRequires:	perl(IO::Socket::INET)
BuildRequires:	perl(Net::Config)
BuildRequires:	perl(Net::FTP)
BuildRequires:	perl(Net::HTTP)
BuildRequires:	perl(Net::NNTP)
BuildRequires:	perl(overload)
BuildRequires:	perl(Scalar::Util)
BuildRequires:	perl(Socket)
BuildRequires:	perl(Storable) >= 2.15
BuildRequires:	perl(warnings)
BuildRequires:	perl(XSLoader)
# Test Suite
# (no additional dependencies)
# Runtime
Requires:	perl(:MODULE_COMPAT_%(eval "`perl -V:version`"; echo $version))
Requires:	perl(AnyEvent) >= 7
Requires:	perl(AnyEvent::AIO) >= 1
Requires:	perl(AnyEvent::BDB) >= 1
Requires:	perl(EV) >= 4
Requires:	perl(Event) >= 1.08
Requires:	perl(Guard) >= 0.5
Requires:	perl(Storable) >= 2.15

# Don't provide private shared objects, or the redundant Time::HiRes dependency
%{?perl_default_filter}
%global __provides_exclude ^(perl\\(Coro\\)$|(EV|Event|State)\\.so)

%description
This module collection manages continuations in general, most often in the
form of cooperative threads (also called coros, or simply "coro" in the
documentation). They are similar to kernel threads but don't (in general) run
in parallel at the same time even on SMP machines. The specific flavor of
thread offered by this module also guarantees you that it will not switch
between threads unless necessary, at easily-identified points in your
program, so locking and parallel access are rarely an issue, making thread
programming much safer and easier than using other thread models.

%prep
%setup -q -n Coro-%{version}%{?extraversion}

# Use ucontext backend on non-x86 (setjmp didn't work on s390(x))
%ifnarch %{ix86} x86_64 %{?arm}
%patch0 -p1 -b .ucontext-default
%endif

# Fix shellbangs
sed -i -e '/^#!/ s|.*|#!/usr/bin/perl|' eg/myhttpd
%patch1 -p1 -b .jitpath

# SVt_LAST not declared in perl 5.8
%patch2

%if ! %{rpm49}
# Filter out redundant unversioned provides if we don't have %%perl_default_filter
%global provfilt /bin/sh -c "%{__perl_provides} | grep -Fvx 'perl(Coro)'"
%define __perl_provides %{provfilt}
%endif

%build
# Interactive configuration - use default values
perl Makefile.PL INSTALLDIRS=vendor OPTIMIZE="%{optflags}" < /dev/null
make %{?_smp_mflags}

%install
rm -rf %{buildroot}
make pure_install DESTDIR=%{buildroot}
find %{buildroot} -type f -name .packlist -exec rm -f {} ';'
find %{buildroot} -type f -name '*.bs' -a -size 0 -exec rm -f {} ';'
%{_fixperms} %{buildroot}

%check
make test

%clean
rm -rf %{buildroot}

%files
%if 0%{?_licensedir:1}
%license COPYING
%else
%doc COPYING
%endif
%doc Changes README README.linux-glibc
%doc doc/* eg/
%{perl_vendorarch}/auto/Coro/
%{perl_vendorarch}/Coro.pm
%{perl_vendorarch}/Coro/
%{_mandir}/man3/Coro.3*
%{_mandir}/man3/Coro::AIO.3*
%{_mandir}/man3/Coro::AnyEvent.3*
%{_mandir}/man3/Coro::BDB.3*
%{_mandir}/man3/Coro::Channel.3*
%{_mandir}/man3/Coro::Debug.3*
%{_mandir}/man3/Coro::EV.3*
%{_mandir}/man3/Coro::Event.3*
%{_mandir}/man3/Coro::Handle.3*
%{_mandir}/man3/Coro::Intro.3*
%{_mandir}/man3/Coro::LWP.3*
%{_mandir}/man3/Coro::MakeMaker.3*
%{_mandir}/man3/Coro::RWLock.3*
%{_mandir}/man3/Coro::Select.3*
%{_mandir}/man3/Coro::Semaphore.3*
%{_mandir}/man3/Coro::SemaphoreSet.3*
%{_mandir}/man3/Coro::Signal.3*
%{_mandir}/man3/Coro::Socket.3*
%{_mandir}/man3/Coro::Specific.3*
%{_mandir}/man3/Coro::State.3*
%{_mandir}/man3/Coro::Storable.3*
%{_mandir}/man3/Coro::Timer.3*
%{_mandir}/man3/Coro::Util.3*

%changelog
* Sun Oct  4 2015 Paul Howarth <paul@city-fan.org> - 6.48-1
- Update to 6.48
  - Fix memory leak when Coro::Handle uses Coro::EV internally
  - Update code to libev 4 API (internally, Coro still used the version 3 API calls)
  - Remove unused hv_sig
- Explicitly BR: perl-devel, needed for EXTERN.h

* Sun Jul 12 2015 Paul Howarth <paul@city-fan.org> - 6.47-1
- Update to 6.47
  - swap_svs - NVs can be in the SV head beginning with 5.22
- Add patch to support building with perl 5.8.8 (missing declaration of
  SVt_LAST)

* Tue Jun 30 2015 Paul Howarth <paul@city-fan.org> - 6.46-1
- Update to 6.46
  - Avoid segfaulting (or worse) when tracing a cancelled thread
  - The Event module silently broke its public hook API, causing failures on
    perls compiled with -Duselongdouble; now requiring the latest version of
    Event because I don't know which release changed this (it's not mentioned
    in the ChangeLog)
  - Allow xs level enter/leave hooks via CoroAPI

* Sat Jun 27 2015 Paul Howarth <paul@city-fan.org> - 6.43-1
- Update to 6.43
  - Use stability canary
  - Port to stableperl-5.22.0-1.001
  - Update libecb

* Thu Feb 12 2015 Paul Howarth <paul@city-fan.org> - 6.42-1
- Update to 6.42
  - Coro::SemaphoreSet->try did not actually work
  - Upgrade libecb to be C11 compliant
- Classify buildreqs by usage

* Mon Sep 22 2014 Paul Howarth <paul@city-fan.org> - 6.41-1
- Update to 6.41
  - Restore portability to perl 5.8.x
  - Give new Coro's a valid GvHV(PL_hintgv) - this is slow and takes up some
    memory, but fixes "use feature" and similar modules when used inside a Coro
  - Allow zero as argument to Coro::Channel to mean the same thing as no
    argument; this works with older versions as well, but wasn't legal until
    now
  - Slightly better C header file detection

* Fri Sep 12 2014 Paul Howarth <paul@city-fan.org> - 6.39-4
- Use %%license where possible

* Tue Jun 10 2014 Paul Howarth <paul@city-fan.org> - 6.39-1
- Update to 6.39
  - Check that perl slots actually have enough space to hold interpreter
    variables
  - Untested port to perl 5.19 (64 bit tmps indices)
  - croak when cancel is called without a thread context
  - Work around more incompatible changes in 5.20
- BR: Perl 5.10 or later as Coro fails to run on Perl 5.8 now

* Tue Mar  4 2014 Paul Howarth <paul@city-fan.org> - 6.37-1
- Update to 6.37
  - Removed left-over debugging code from debugging a perl bug, of all things

* Tue Mar  4 2014 Paul Howarth <paul@city-fan.org> - 6.36-1
- Update to 6.36
  - Semaphores would not clear the destroy hook when interrupted by ->throw,
    causing segfaults or worse
  - ->throw on a thread waiting for a semaphore did not acquire the semaphore,
    but also didn't wake up other waiters, possibly causing a deadlock
  - "FATAL: $Coro::IDLE blocked itself" will now use Carp::confess to report a
    full stacktrace, which should help find out where the actual call is
  - "Atomically" destroy data in slf_destroy, because it is the right thing to
    do, just in case
  - Disable ecb.h libm dependency, to compile on ancient systems or under
    adverse conditions

* Mon Nov 18 2013 Paul Howarth <paul@city-fan.org> - 6.33-1
- Update to 6.33
  - Do not crash when freeing padlists with holes (in 5.18)
  - Tentative SVt_BIND 5.19 port/fix

* Tue Nov  5 2013 Paul Howarth <paul@city-fan.org> - 6.32-1
- Update to 6.32
  - Use a new algorithm to derive padlists for perl 5.18; the old one could
    lead to 0-pointer accesses inside perl

* Tue Jul 23 2013 Paul Howarth <paul@city-fan.org> - 6.31-2
- Perl 5.18 rebuild

* Thu May  9 2013 Paul Howarth <paul@city-fan.org> - 6.31-1
- Update to 6.31
  - Coro::AIO requests would crash if the thread was ready'd while the request
    was ongoing

* Wed May  8 2013 Paul Howarth <paul@city-fan.org> - 6.29-1
- Update to 6.29
  - Clean remnants of existing __DIE__ and __WARN__ handlers so they lose their
    magic and will not cause segfaults later
  - Improved Coro::State documentation a bit
  - Coro::Debug::command now flushes the output
  - Add hack detection code for x32 abi, because the braindead slugs who
    designed that made it look exactly like x86_64 without providing proper
    compile-time symbols to test for it; as a result, this detection cannot
    work reliably
  - valgrind stack registering was broken
  - Do not rely on Time::HiRes anymore in Coro::Debug
  - When an on_destroy handler destructs the coro currently being destructed a
    perl scalar could be accessed after being freed, likely causing a crash
- Drop perl_free() segfault patch, no longer needed

* Sat Dec  8 2012 Paul Howarth <paul@city-fan.org> - 6.23-1
- Update to 6.23
  - Use experimental fiber implementation on native Windows perls
  - Use sizeof (void *) as multiplication factor for stack sizes, to accomodate
    the totally braindamaged Microsoft 64 bit "os"
  - Changed verifier host from win2k-ap510-32 to win7-sp516-32/64; activeperl
    5.16 crashes when PerlIO_define_layer is called due to some bug in the perl
    dll, strawberry perl at least passes the test suite
  - Implement Coro::Handle->peeraddr/host/port, for slightly improved
    compatibility with LWP
  - Implement 5.17 compatibility by almost blindly applying a good-looking
    patch by Father Chrysostomos
  - Move stack management functions into libcoro 3
  - libcoro version 3 "released"
  - Support magic values as timed_io_once args
  - Recommend AnyEvent 7+ or EV 4+; also require EV version 4 or newer for
    Coro::EV

* Wed Oct 10 2012 Paul Howarth <paul@city-fan.org> - 6.10-1
- Update to 6.10
  - Fix building on big-endian systems
  - Disable memory fences in ecb.h to improve portability

* Sun Oct  7 2012 Paul Howarth <paul@city-fan.org> - 6.09-1
- Update to 6.09
  - Coro::EV I/O watchers were not interruptible by exceptions
    (Coro::State::throw)
  - ->throw now puts threads into the ready queue, as this seems to be expected
    by existing code, and code that doesn't cope with spurious wakeups needs
    fixing anyway
  - Use fd -1 in mmap
  - Cast I32 to int in error message printf
  - Warn about broken so-called "hardened" kernels

* Wed Jul 11 2012 Paul Howarth <paul@city-fan.org> - 6.08-2
- BR: perl(base), perl(Carp), perl(Exporter), perl(IO::Socket::INET),
  perl(Net::Config), perl(Net::HTTP), perl(Net::FTP), perl(Net::NNTP),
  perl(Socket), perl(Time::HiRes) and perl(XSLoader)
- BR: /usr/include/valgrind/valgrind.h for valgrind support

* Fri Apr 13 2012 Paul Howarth <paul@city-fan.org> - 6.08-1
- Update to 6.08
  - Be more aggressive about exiting like perl does - formerly, exiting from
    the non-main thread would not execute END blocks
- Add patch to work around segfault in perl_free() on perl 5.10.0 on x86_64
- Drop %%defattr, redundant since rpm 4.4
- Don't need to remove empty directories from buildroot

* Sat Nov 12 2011 Paul Howarth <paul@city-fan.org> - 6.07-1
- Update to 6.07
  - Work around a bug in PerlIO (setting $SIG{__WARN__} to a PVCV)
  - Update ecb.h

* Thu Nov  3 2011 Paul Howarth <paul@city-fan.org> - 6.06-2
- Fix ucontext on ARM (#750805)

* Tue Aug  9 2011 Paul Howarth <paul@city-fan.org> - 6.06-1
- Update to 6.06
  - Cygwin unfortunately patches the stack at runtime, so we use the pthreads
    backend, which is an order of magnitude slower; unfortunately, Cygwin's
    pthread implementation isn't very complete either, so allocate the stack
    twice just to be sure

* Fri Aug  5 2011 Paul Howarth <paul@city-fan.org> - 6.05-1
- Update to 6.05
  - Blush: condvar values would not be propagated from send to recv anymore
  - Use exponential increase for the readline buffer length in Coro::Handle;
    also reduce initial allocation to 1020 from 4096 bytes

* Wed Aug  3 2011 Paul Howarth <paul@city-fan.org> - 6.04-1
- Update to 6.04
  - Use even more efficient and more compatible condvars for compatibility to
    AnyEvent 6.x :)
  - More inconsequential ecb.h updates

* Wed Aug  3 2011 Paul Howarth <paul@city-fan.org> - 6.03-1
- Update to 6.03
  - Change how Coro patches AnyEvent condvars for compatibility with
    AnyEvent 6.x
  - Update ecb.h to no longer include <pthread.h> in case WinNT.h hasn't been
    included
- Add rpm 4.9 native filtering

* Wed Jul 13 2011 Paul Howarth <paul@city-fan.org> - 6.02-2
- Rebuild for perl 5.14.1 in Rawhide

* Wed Jul 13 2011 Paul Howarth <paul@city-fan.org> - 6.02-1
- Update to 6.02
  - Improve portability to Gentoo - Gentoo manages to put perl variables in
    memory areas that are farther than 2 Gb apart, which the jit couldn't
    handle and barfed; now it's just a bit slower on Gentoo and similar systems
- Fix dist tag for CentOS 6 and Scientific Linux

* Sun Jul  3 2011 Paul Howarth <paul@city-fan.org> - 6.01-1
- Update to 6.01
  - Workarounds are good, but the test for whether pthreads are used was not
    good; this one should be better
  - check differently whether gcc generates cfi instructions itself
- Drop cfi patch, no longer needed

* Wed Jun 29 2011 Paul Howarth <paul@city-fan.org> - 6.0-1
- Update to 6.0
  - INCOMPATIBLE CHANGE: unreferenced coro objects will now be destroyed and
    cleaned up automatically (e.g. async { schedule })
  - Implement a JIT compiler for part of the thread switch code, which gives a
    50%% speed improvement on threaded perls, and about 4%% on non-threaded
    perls (so threaded perls now finally reach about half the speed of
    non-threaded perls)
  - Slightly modernise Coro::Intro, add section about rouse functions
  - avoid DEFSV and ERRSV, giving another 10%% improvement in thread switching
  - Coro::State->is_destroyed is now called is_zombie
  - Implement a Coro->safe_cancel method that might fail, but cancels in a
    "safer" way if it succeeds
  - Add preliminary support for DEBUGGING perls
  - Get rid of two hash-accesses when initialising a new Coro - this speeds up
    coro creation by almost a factor of two
  - Croak when a coro that is being cancelled tries to block (e.g. while
    executing a guard block), instead of crashing or deadlocking
  - Use a more robust and also faster method to identify Coro::State objects -
    speeds up everything a bit
  - Implement Coro->cancel in XS for a 20%% speed improvement, and to be able to
    implement mutual cancellation
  - Speed up context switches by a percent or two by more efficiently
    allocating context stack entries
  - Implement Coro->join and Coro->on_destroy in XS for a speedup and a
    reduction in memory use
  - Cancelling a coro while it itself is cancelling another coro is now
    supported and working, instead of triggering an assertion
  - Be a bit more crash-resistant when calling (buggy) on_destroy callbacks
    (best effort)
  - Move on_destroy into the slf_frame, to allow extension slf functions to
    have destructors
  - Get rid of coro refcounting - simply crash in other interpreter threads by
    nulling the pointers on clone
  - Simplify warn/die hook handling when loading Coro - the convoluted logic
    seems no longer to be neccessary
  - Use libecb instead of our own home-grown gcc hacks
  - Document alternatives to Coro::LWP - please use them :)
  - Work around another mindless idiotic NEEDLESS bug in openbsd/mirbsds
    sigaltstack
- Add patch to remove CFI assembler code if we have too old an assembler
- Add versioned dependency perl(EV) >= 3
- Filter dependency on Time::HiRes since we always have gettimeofday
- Use DESTDIR rather than PERL_INSTALL_ROOT
- Install to vendor directories
- Nobody else likes macros for commands

* Wed Feb 23 2011 Paul Howarth <paul@city-fan.org> - 5.37-2
- Update to 5.372
  - Backport to windows process emulation code again
  - Fix some portability issues with Time::HiRes
- Drop State.xs syntax error patch, applied upstream (CPAN RT#65991)

* Mon Feb 21 2011 Paul Howarth <paul@city-fan.org> - 5.37-1
- Update to 5.37
  - Add a big "Coro thread life cycle" section to "man Coro"
  - Try a tentative workaround against the breakage that 5.13 has introduced
    without deprecation period
  - No longer use Time::HiRes if gettimeofday is available, which saves quite
    a lot of memory
- Fix State.xs syntax (RT#65991)
- Drop redundant buildreq perl(Time::HiRes)

* Tue Feb 15 2011 Paul Howarth <paul@city-fan.org> - 5.26-1
- Update to 5.26
  - Automatically load Coro::Channel, Coro::RWLock, Coro::Semaphore,
    Coro::SemaphoreSet, Coro::Signal and Coro::Specific on first "new" method
    call
  - Undocument Coro::Timer::sleep and obsolete whole module
  - Optimize Coro::Timer::timeout memory and CPU usage
  - Slightly updated Coro::Intro for recent changes
  - Do not initialise PL_dirty anymore

* Wed Jan 26 2011 Paul Howarth <paul@city-fan.org> - 5.25-5
- BR: perl(EV) for Coro::EV support

* Mon Jan 24 2011 Paul Howarth <paul@city-fan.org> - 5.25-4
- Import from Fedora

* Fri Jan 21 2011 Mathieu Bridon <bochecha@fedoraproject.org> - 5.25-3
- Allow building on systems without %%fix_shbang_line macro (needed for EL6)

* Mon Jan 10 2011 Dan Horák <dan[at]danny.cz> - 5.25-2
- Use ucontext backend on non-x86

* Tue Jan 04 2011 Petr Pisar <ppisar@redhat.com> - 5.25-1
- 5.25 import
- Disable perl(EV) support as it's not packaged yet
